package ligneproduit;

import java.util.Collection;


/**
 * Interface représentant une facture.
 * <p> Invariant(s) (il concerne théoriquement la méthode getLignes(), mais vous avez intérêt à 
 * le prendre en compte dès le début) : quand on appelle getLigne(), on veut récupérer une seule ligne de facture 
 * par produit.
 * <p> Les lignes de facture ne doivent contenir que des produits commandés en au moins 1 exemplaire. 
 * @author rosmord
 */
public class Facture {
	
	/**
	 * La classe facture DOIT avoir un constructeur par défaut, qui construit une facture "vide".
	 */
	public Facture() {
		// à compléter si besoin...
	}
	
	/**
	 * Ajoute un produit à la facture, dans une certaine quantité.
	 * <p> Pre condition : quantite supérieure ou égale à 0.
	 * <p> Post-condition: 
	 * <ul>
	 * <li>la quantité du produit commandé est <em>augmentée</em> de la valeur 
	 * passée en paramètre. 
	 * <li> Si le produit était déjà commandé, les quantités s'ajoutent ! 
	 * </ul>
	 * 
	 * @param p le produit à ajouter
	 * @param quantite la quantité (doit être positive).
	 * @throws IllegalArgumentException si quantite est négative.
	 */
	public void ajouter(Produit p, int quantite) {
		throw new RuntimeException("Enlevez cette ligne et écrivez-moi !!!");
	}
	
	/**
	 * Retire une certaine quantité d'un produit de la facture.
	 * <p> Pré-condition : qte &ge; 0 et qte inférieure ou égale à la quantité déjà commandée du produit p.   
	 * <p> Post-condition: la quantité est commandée du produit est diminuée de qte. Si cela la fait passer
	 * à 0, la ligne concernant le produit doit être retirée de la facture. 
	 * @param p un produit
	 * @param qte une quantité à retirer.
	 * @throws IllegalArgumentException si qte supérieure à la quantité déjà commandée.
	 */
	public void retirer(Produit p, int qte) {
		throw new RuntimeException("Enlevez cette ligne et écrivez-moi !!!");
	}
	
	/**
	 * Retourne les lignes de la facture. 
	 * <p> En temps normal, on devrait s'arranger pour que cette 
	 * collection soit une copie (ne permette pas la modification
	 * réelle de la facture, seulement sa consultation).
	 * Pour cet exercice, on négligera ce problème.</p>
	 * <p> Voir le commentaire en début d'interface pour la définition de getLignes().
	 * @return
	 */
	public Collection<LigneFacture> getLignes() {
		throw new RuntimeException("Enlevez cette ligne et écrivez-moi !!!");
	}
	
	/**
	 * Retourne la quantité de produit commandé pour un produit p.
	 * @param p le produit.
	 * @return la quantité de p commandée (0 si le produit n'est pas dans la facture).
	 */
	public int getQuantite(Produit p) {
		throw new RuntimeException("Enlevez cette ligne et écrivez-moi !!!");
	}
	
	/**
	 * Retourne le montant total de la facture. 
	 * @return
	 */
	public double getTotal() {
		throw new RuntimeException("Enlevez cette ligne et écrivez-moi !!!");
	}
	
}
